@@ -3,6 +3,7 @@ import datetime
 import gettext
 import logging
 import gc
+import time
 
 from gi.repository import GLib
 from gi.repository import GObject
@@ -61,9 +62,10 @@ class JobDocThumbnailer(Job):
     SMALL_THUMBNAIL_WIDTH = 64
     SMALL_THUMBNAIL_HEIGHT = 80
 
-    def __init__(self, factory, id, doclist):
+    def __init__(self, factory, id, doclist, already_loaded):
         Job.__init__(self, factory, id)
-        self.__doclist = doclist
+        self.doclist = doclist
+        self.__already_loaded = already_loaded
         self.__current_idx = -1
 
     def __resize(self, img):
@@ -86,26 +88,37 @@ class JobDocThumbnailer(Job):
 
     def do(self):
         self.can_run = True
-        if self.__current_idx >= len(self.__doclist):
+        if self.__current_idx >= len(self.doclist):
             return
         if not self.can_run:
+            logger.info(
+                "Thumbnailing [%s] interrupted (not even started yet)", self
+            )
             return
 
         if self.__current_idx < 0:
+            logger.info(
+                "Start thumbnailing %d documents (%s)",
+                len(self.doclist), self
+            )
             self.emit('doc-thumbnailing-start')
             self.__current_idx = 0
 
-        for idx in range(self.__current_idx, len(self.__doclist)):
-            doc = self.__doclist[idx]
+        for (idx, doc) in enumerate(self.doclist):
+            if doc.docid in self.__already_loaded:
+                continue
+
             if doc.nb_pages <= 0:
                 continue
 
+            start = time.time()
+
             # always request the same size, even for small thumbnails
             # so we don't invalidate cache + previous thumbnails
             img = doc.pages[0].get_thumbnail(BasicPage.DEFAULT_THUMB_WIDTH,
                                              BasicPage.DEFAULT_THUMB_HEIGHT)
-            doc.drop_cache()
             if not self.can_run:
+                logger.info("Thumbnailing [%s] interrupted (0)", self)
                 return
 
             (w, h) = img.size
@@ -117,23 +130,33 @@ class JobDocThumbnailer(Job):
             h /= factor
             img = img.resize((int(w), int(h)), PIL.Image.ANTIALIAS)
             if not self.can_run:
+                logger.info("Thumbnailing [%s] interrupted (1)", self)
                 return
 
             img = self.__resize(img)
             if not self.can_run:
+                logger.info("Thumbnailing [%s] interrupted (2)", self)
                 return
 
             img = add_img_border(img, width=self.THUMB_BORDER)
             if not self.can_run:
+                logger.info("Thumbnailing [%s] interrupted (3)", self)
                 return
 
             pixbuf = image2pixbuf(img)
 
+            stop = time.time()
+
+            logger.info(
+                "[%s]: Got thumbnail for %s (%fs)",
+                self, doc.docid, stop - start
+            )
             self.emit('doc-thumbnailing-doc-done', pixbuf, doc,
-                      idx, len(self.__doclist))
+                      idx, len(self.doclist))
 
             self.__current_idx = idx
 
+        logger.info("End of thumbnailing [%s]", self)
         self.emit('doc-thumbnailing-end')
 
     def stop(self, will_resume=False):
@@ -151,13 +174,15 @@ class JobFactoryDocThumbnailer(JobFactory):
         JobFactory.__init__(self, "DocThumbnailer")
         self.__doclist = doclist
 
-    def make(self, doclist):
+    def make(self, doclist, already_loaded=()):
         """
         Arguments:
             doclist --- must be an array of (position, document), position
                         being the position of the document
         """
-        job = JobDocThumbnailer(self, next(self.id_generator), doclist)
+        job = JobDocThumbnailer(
+            self, next(self.id_generator), doclist, already_loaded
+        )
         job.connect(
             'doc-thumbnailing-start',
             lambda thumbnailer:
@@ -201,7 +226,7 @@ class JobLabelCreator(Job):
 
     def do(self):
         self.emit('label-creation-start')
-        self._wait(0.5) # give a little bit of time to Gtk to update
+        self._wait(0.5)  # give a little bit of time to Gtk to update
         try:
             self.__docsearch.create_label(self.__new_label, self.__doc,
                                           self.__progress_cb)
@@ -262,7 +287,7 @@ class JobLabelUpdater(Job):
 
     def do(self):
         self.emit('label-updating-start')
-        self._wait(0.5) # give a little bit of time to Gtk to update
+        self._wait(0.5)  # give a little bit of time to Gtk to update
         try:
             self.__docsearch.update_label(self.__old_label, self.__new_label,
                                           self.__progress_cb)
@@ -322,7 +347,7 @@ class JobLabelDeleter(Job):
 
     def do(self):
         self.emit('label-deletion-start')
-        self._wait(0.5) # give a little bit of time to Gtk to update
+        self._wait(0.5)  # give a little bit of time to Gtk to update
         try:
             self.__docsearch.destroy_label(self.__label, self.__progress_cb)
         finally:
@@ -472,13 +497,14 @@ class ActionCreateLabel(SimpleAction):
         SimpleAction.__init__(self, "Creating label")
         self.__main_win = main_window
         self.__doc_properties = doc_properties
+        self._dialog = None  # for tests / automated screenshots only
 
     def do(self):
         SimpleAction.do(self)
-        labeleditor = LabelEditor()
+        self._dialog = labeleditor = LabelEditor()
         if labeleditor.edit(self.__main_win.window) == "ok":
-            logger.info("Adding label %s to doc %s"
-                        % (labeleditor.label.name, self.__main_win.doc))
+            logger.info("Adding label %s to doc %s",
+                        labeleditor.label.name, self.__main_win.doc)
             job = self.__doc_properties.job_factories['label_creator'].make(
                 self.__main_win.docsearch, labeleditor.label,
                 self.__main_win.doc)
@@ -494,6 +520,7 @@ class ActionEditLabel(SimpleAction):
         SimpleAction.__init__(self, "Editing label")
         self.__main_win = main_window
         self.__doc_properties = doc_properties
+        self._dialog = None  # for tests / automated screenshots only
 
     def do(self):
         SimpleAction.do(self)
@@ -510,7 +537,7 @@ class ActionEditLabel(SimpleAction):
         label = Label(label_name, label_color)
 
         new_label = copy(label)
-        editor = LabelEditor(new_label)
+        self._dialog = editor = LabelEditor(new_label)
         reply = editor.edit(self.__main_win.window)
         if reply != "ok":
             logger.warning("Label edition cancelled")
@@ -530,6 +557,7 @@ class ActionDeleteLabel(SimpleAction):
         SimpleAction.__init__(self, "Deleting label")
         self.__main_win = main_window
         self.__doc_properties = doc_properties
+        self._label = None
 
     def do(self):
         SimpleAction.do(self)
@@ -543,14 +571,18 @@ class ActionDeleteLabel(SimpleAction):
         label_box = selected_row.get_children()[0]
         label_name = label_box.get_children()[2].get_text()
         label_color = label_box.get_children()[1].get_rgba().to_string()
-        label = Label(label_name, label_color)
+        self._label = Label(label_name, label_color)
 
-        if not ask_confirmation(self.__main_win.window):
+        if not ask_confirmation(self.__main_win.window, self._do_delete):
             return
 
-        logger.info("Label {} must be deleted. Applying changes".format(label))
+    def _do_delete(self):
+        logger.info("Label {} must be deleted. Applying changes".format(
+            self._label
+        ))
         job = self.__doc_properties.job_factories['label_deleter'].make(
-            self.__main_win.docsearch, label)
+            self.__main_win.docsearch, self._label
+        )
         self.__main_win.schedulers['main'].schedule(job)
         self.__doc_properties.refresh_label_list()
 
@@ -566,11 +598,9 @@ class ActionDeleteDoc(SimpleAction):
         Ask for confirmation and then delete the document being viewed.
         """
         super().do()
-        if not ask_confirmation(self.__main_win.window):
-            return
-        GLib.idle_add(self._do)
+        ask_confirmation(self.__main_win.window, self._do_delete)
 
-    def _do(self):
+    def _do_delete(self):
         SimpleAction.do(self)
         if self.__doc is None:
             doc = self.__main_win.doc
@@ -596,11 +626,15 @@ class ActionDeleteDoc(SimpleAction):
         self.__main_win.schedulers['main'].schedule(job)
 
     def _on_doc_deleted_from_index(self, doc):
+        # Windows: all file descriptors to the document must be closed
+        self.__main_win.docsearch.gc()
         doc.destroy()
         self.__main_win.refresh_doc_list()
 
 
 class DocList(object):
+    LIST_CHUNK = 50
+
     def __init__(self, main_win, config, widget_tree):
         self.__main_win = main_win
         self.__config = config
@@ -611,15 +645,20 @@ class DocList(object):
             JobDocThumbnailer.SMALL_THUMBNAIL_HEIGHT)
 
         self.gui = {
+            'headerbar': widget_tree.get_object("doclist_headerbar_left"),
+            'nb_results': widget_tree.get_object("searchGlobalNbResults"),
             'list': widget_tree.get_object("listboxDocList"),
             'box': widget_tree.get_object("doclist_box"),
             'scrollbars': widget_tree.get_object("scrolledwindowDocList"),
+            'last_scrollbar_value': -1,
             'spinner': SpinnerAnimation((0, 0)),
+            'nb_boxes': 0,
         }
-        self.gui['loading'] = Canvas(self.gui['scrollbars'])
-        self.gui['loading'].set_visible(False)
-        self.gui['box'].add(self.gui['loading'])
-        self.gui['scrollbars'].connect(
+        loading_box = widget_tree.get_object("scrolledwindowLoading")
+        self.gui['loading'] = Canvas(loading_box)
+        self.gui['loading'].set_visible(True)
+        loading_box.add(self.gui['loading'])
+        loading_box.connect(
             "size-allocate",
             lambda x, s: GLib.idle_add(self._on_size_allocate)
         )
@@ -636,12 +675,16 @@ class DocList(object):
 
         self.model = {
             'has_new': False,
+            'docids': [],
             'by_row': {},  # Gtk.ListBoxRow: docid
             'by_id': {},  # docid: Gtk.ListBoxRow
             # keep the thumbnails in cache
             'thumbnails': {}  # docid: pixbuf
         }
-        self.new_doc = ImgDoc(config['workdir'].value)
+        workdir = self.__main_win.docsearch.fs.safe(
+            config['workdir'].value
+        )
+        self.new_doc = ImgDoc(self.__main_win.docsearch.fs, workdir)
 
         self.job_factories = {
             'doc_thumbnailer': JobFactoryDocThumbnailer(self),
@@ -652,16 +695,17 @@ class DocList(object):
             "value-changed",
             lambda v: GLib.idle_add(self._on_scrollbar_value_changed)
         )
+        self.gui['scrollbars'].connect(
+            "size-allocate",
+            lambda w, s: GLib.idle_add(
+                self._on_scrollbar_value_changed, True
+            )
+        )
 
         self.gui['list'].add_events(Gdk.EventMask.BUTTON_PRESS_MASK)
 
         self.gui['list'].connect("button-press-event", self._on_button_pressed)
 
-        self.gui['list'].connect(
-            "size-allocate",
-            lambda w, s: GLib.idle_add(self._on_scrollbar_value_changed)
-        )
-
         self.gui['list'].connect("drag-motion", self._on_drag_motion)
         self.gui['list'].connect("drag-leave", self._on_drag_leave)
         self.gui['list'].connect(
@@ -694,22 +738,42 @@ class DocList(object):
             event.state & Gdk.ModifierType.SHIFT_MASK
         )
 
-    def _on_scrollbar_value_changed(self):
+    def _on_scrollbar_value_changed(self, force=False):
+        logger.debug("[thumbnailing] scrollbar values changed")
         vadjustment = self.gui['scrollbars'].get_vadjustment()
+        if (not force and
+                vadjustment.get_value() ==
+                self.gui['last_scrollbar_value']):
+            logger.debug(
+                "[thumbnailing] scrollbar position unchanged --> ignored"
+            )
+            # avoid repeated requests due to adding documents to the
+            # GtkListBox (even if frozen ...)
+            return
+        self.gui['last_scrollbar_value'] = vadjustment.get_value()
         self.__main_win.schedulers['main'].cancel_all(
             self.job_factories['doc_thumbnailer']
         )
 
         # because we use GLib.idle_add() before calling this function
-        # the scrollbar may have disappear (--> application close)
+        # the scrollbar may have disappeared (application being closed)
         # before we are called
         if vadjustment is None:
+            logger.debug("[thumbnailing] no scrollbar.vadjustment")
             return
 
         # XXX(Jflesch): assumptions: values are in px
         value = vadjustment.get_value()
+        lower = vadjustment.get_lower()
+        upper = vadjustment.get_upper()
         page_size = vadjustment.get_page_size()
 
+        # if we are in the lower part (10%), add the next chunk of boxes
+        u = upper - lower
+        v = value - lower + page_size
+        if lower + page_size < upper and v >= u:
+            self._add_boxes()
+
         start_y = value
         end_y = value + page_size
 
@@ -724,6 +788,7 @@ class DocList(object):
             end_idx = end_row.get_index()
         if start_row == end_row:
             return
+        logger.debug("Thumbnailing: {} - {}".format(start_idx, end_idx))
         if end_idx < start_idx:
             logger.warn("Thumbnailing: End_idx (%d) < start_idx (%d) !?"
                         % (end_idx, start_idx))
@@ -747,10 +812,12 @@ class DocList(object):
         if len(documents) > 0:
             logger.info("Will get thumbnails for %d documents [%d-%d]"
                         % (len(documents), start_idx, end_idx))
-            job = self.job_factories['doc_thumbnailer'].make(documents)
+            job = self.job_factories['doc_thumbnailer'].make(
+                documents, self.model['thumbnails']
+            )
             self.__main_win.schedulers['main'].schedule(job)
 
-    def _scroll_to(self, row):
+    def scroll_to(self, row):
         adj = row.get_allocation().y
         adj -= (self.gui['scrollbars'].get_vadjustment().get_page_size() / 2)
         adj += (row.get_allocation().height / 2)
@@ -821,7 +888,6 @@ class DocList(object):
 
         target_doc.add_page(src_page.img, src_page.boxes)
         src_page.destroy()
-        src_page.doc.drop_cache()
         if src_page.doc.nb_pages <= 0:
             src_page.doc.destroy()
         drag_context.finish(True, True, time)  # success = True
@@ -847,7 +913,10 @@ class DocList(object):
     def get_new_doc(self):
         if self.new_doc.is_new:
             return self.new_doc
-        self.new_doc = ImgDoc(self.__config['workdir'].value)
+        workdir = self.__main_win.docsearch.fs.safe(
+            self.__config['workdir'].value
+        )
+        self.new_doc = ImgDoc(self.__main_win.docsearch.fs, workdir)
         return self.new_doc
 
     def insert_new_doc(self):
@@ -870,6 +939,9 @@ class DocList(object):
         doc = self.get_new_doc()
         self.__main_win.show_doc(doc)
 
+    def unselect_all(self):
+        self.gui['list'].unselect_all()
+
     def clear(self):
         self.gui['list'].freeze_child_notify()
         try:
@@ -882,6 +954,7 @@ class DocList(object):
             self.model['by_row'] = {}
             self.model['by_id'] = {}
             self.model['has_new'] = False
+            self.gui['nb_boxes'] = 0
         finally:
             self.gui['list'].thaw_child_notify()
 
@@ -958,30 +1031,56 @@ class DocList(object):
         # remove the list, put the canvas+spinner instead
         self.gui['loading'].remove_all_drawers()
         self.gui['loading'].add_drawer(self.gui['spinner'])
-        self.gui['list'].set_visible(False)
-        self.gui['loading'].set_visible(True)
+        self.__main_win.switch_leftpane('loading')
         self._on_size_allocate()
 
     def show_loading(self):
         GLib.idle_add(self._show_loading)
 
-    def set_docs(self, documents, need_new_doc=True):
+    def _add_boxes(self):
+        start = self.gui['nb_boxes']
+        stop = self.gui['nb_boxes'] + self.LIST_CHUNK
+        docs = self.model['docids'][start:stop]
+
+        logger.info("Will display documents {}:{} (actual number: {})".format(
+            start, stop, len(docs)
+        ))
+
+        for docid in docs:
+            rowbox = self.model['by_id'][docid]
+            self.gui['list'].add(rowbox)
+            self.gui['nb_boxes'] += 1
+
+    def set_docs(self, documents, all_docs=True):
         self.__main_win.schedulers['main'].cancel_all(
             self.job_factories['doc_thumbnailer']
         )
 
         self.clear()
+        GLib.idle_add(self._set_docs, documents, all_docs)
+
+    def _set_docs(self, documents, all_docs=True):
+        need_new_doc = all_docs
+        display_count = not all_docs
+
+        title = _("Documents") if all_docs else _("Matching papers")
+        self.gui['headerbar'].set_title(title)
+
+        self.gui['nb_results'].set_visible(display_count)
+        if display_count:
+            self.gui['nb_results'].set_text(str(len(documents)))
+
+        self.model['docids'] = [doc.docid for doc in documents]
+        for doc in documents:
+            rowbox = Gtk.ListBoxRow()
+            selected = (doc.docid == self.__main_win.doc.docid)
+            self._make_listboxrow_doc_widget(doc, rowbox, selected)
+            self.model['by_row'][rowbox] = doc.docid
+            self.model['by_id'][doc.docid] = rowbox
 
         self.gui['list'].freeze_child_notify()
         try:
-            for doc in documents:
-                rowbox = Gtk.ListBoxRow()
-                selected = (doc.docid == self.__main_win.doc.docid)
-                self._make_listboxrow_doc_widget(doc, rowbox, selected)
-                self.model['by_row'][rowbox] = doc.docid
-                self.model['by_id'][doc.docid] = rowbox
-                self.gui['list'].add(rowbox)
-
+            self._add_boxes()
             if need_new_doc:
                 self.insert_new_doc()
 
@@ -991,15 +1090,16 @@ class DocList(object):
 
         if (self.__main_win.doc and
                 self.__main_win.doc.docid in self.model['by_id']):
+            self.make_doc_box_visible(self.__main_win.doc)
             row = self.model['by_id'][self.__main_win.doc.docid]
             self.gui['list'].select_row(row)
-            GLib.idle_add(self._scroll_to, row)
+            GLib.idle_add(self.scroll_to, row)
+
+        self.gui['last_scrollbar_value'] = -1  # force refresh of thumbnails
+        GLib.idle_add(self._on_scrollbar_value_changed, True)
 
         # remove the spinner, put the list instead
-        self.gui['loading'].remove_all_drawers()
-        self.gui['loading'].set_visible(False)
-
-        GLib.idle_add(self._on_scrollbar_value_changed)
+        self.__main_win.switch_leftpane("doc_list")
 
     def refresh_docs(self, docs, redo_thumbnails=True):
         """
@@ -1021,7 +1121,11 @@ class DocList(object):
                 return
 
         # and rethumbnail what must be
-        docs = [x for x in docs]
+        if redo_thumbnails:
+            for doc in docs:
+                if doc.docid in self.model['thumbnails']:
+                    self.model['thumbnails'].pop(doc.docid)
+        docs = list(docs)
         logger.info("Will redo thumbnails: %s" % str(docs))
         job = self.job_factories['doc_thumbnailer'].make(docs)
         self.__main_win.schedulers['main'].schedule(job)
@@ -1032,23 +1136,27 @@ class DocList(object):
         the keywords typed by the user in the search field.
         Warning: Will reset all the thumbnail to the default one
         """
-        self.__main_win.schedulers['main'].cancel_all(
+        GLib.idle_add(self._refresh)
+
+    def _refresh(self):
+        self.__main_win.schedulers['search'].cancel_all(
             self.__main_win.job_factories['doc_searcher']
         )
         search = self.__main_win.search_field.get_text()
         self.show_loading()
+        self.__main_win.set_searchbar_visible(search != "")
         job = self.__main_win.job_factories['doc_searcher'].make(
             docsearch=self.__main_win.docsearch,
             sort_func=self.__main_win.get_doc_sorting()[1],
             search_type='fuzzy',
             search=search)
-        self.__main_win.schedulers['main'].schedule(job)
+        self.__main_win.schedulers['search'].schedule(job)
 
     def has_multiselect(self):
         return (len(self.gui['list'].get_selected_rows()) > 1)
 
     def unselect_doc(self, doc):
-        if not doc.docid in self.model['by_id']:
+        if doc.docid not in self.model['by_id']:
             return
         row = self.model['by_id'][doc.docid]
         self.gui['list'].unselect_row(row)
@@ -1068,9 +1176,31 @@ class DocList(object):
         docid = self.model['by_row'][best_row]
         return self.__main_win.docsearch.get_doc_from_docid(docid, inst=False)
 
+    def make_doc_box_visible(self, doc):
+        if doc.docid not in self.model['docids']:
+            return
+        doc_pos = self.model['docids'].index(doc.docid)
+        logger.info(
+            "Document position: {}"
+            " | Number of boxes currently displayed: {}".format(
+                doc_pos, self.gui['nb_boxes']
+            )
+        )
+        if doc_pos < self.gui['nb_boxes']:
+            return
+        self.gui['list'].freeze_child_notify()
+        try:
+            while doc_pos >= self.gui['nb_boxes']:
+                self._add_boxes()
+        finally:
+            self.gui['list'].thaw_child_notify()
+
     def select_doc(self, doc=None, offset=None, open_doc=True):
         self.enabled = open_doc
         try:
+            if doc:
+                self.make_doc_box_visible(doc)
+
             assert(doc is not None or offset is not None)
             if doc is not None:
                 if doc.docid not in self.model['by_id']:
@@ -1093,18 +1223,20 @@ class DocList(object):
                 row = self.gui['list'].get_row_at_index(row_index)
                 if not row:
                     return
+                docid = self.model['by_row'][row]
+                doc = self.__main_win.docsearch.get_doc_from_docid(
+                    docid, inst=True
+                )
+            self.gui['list'].unselect_all()
             self.gui['list'].select_row(row)
             if open_doc and doc:
-                # WORKAROUND(Jflesch): Gtk behavior has changed between
-                # debian stable (jessie) and sid (2016/11/08) ?!
-                # --> we switch document explicitly here but we shouldn't have
-                # to
                 self.__main_win.show_doc(doc)
+            return row
         finally:
             self.enabled = True
 
     def _on_size_allocate(self):
-        visible = self.gui['scrollbars'].get_allocation()
+        visible = self.gui['loading'].get_allocation()
         visible = (visible.width, visible.height)
         self.gui['spinner'].position = (
             (visible[0] - SpinnerAnimation.ICON_SIZE) / 2,
@@ -1286,8 +1418,9 @@ class DocPropertiesPanel(object):
                 buttons=Gtk.ButtonsType.OK,
                 text=msg
             )
-            dialog.run()
-            dialog.destroy()
+            dialog.connect("response", lambda dialog, response:
+                           GLib.idle_add(dialog.destroy))
+            dialog.show_all()
             raise
 
         # Labels
@@ -1322,7 +1455,12 @@ class DocPropertiesPanel(object):
             if has_changed:
                 self.__main_win.upd_index({self.doc})
         else:
-            old_doc = self.doc.clone()
+            old_doc = self.doc
+
+            # Switch to "New document" for now to make sure we lose
+            # all references to the document and pages in the UI/Canvas
+            self.__main_win.actions['new_doc'][1].do()
+
             # this case is more tricky --> del + new
             job = self.__main_win.job_factories['index_updater'].make(
                 self.__main_win.docsearch,
@@ -1337,12 +1475,14 @@ class DocPropertiesPanel(object):
             )
             self.new_doc_date = None
             self.__main_win.schedulers['main'].schedule(job)
-
-        self.__main_win.refresh_header_bar()
+        # XXX(JFlesch)
+        # at this point, be *really* careful to not trigger any operation
+        # that would recreate the document cache : we may be about
+        # to rename the document --> It means deleting it first
+        # --> no file descriptor must be opened on it
 
     def __rename_doc(self, old_doc, new_doc_date):
         old_doc.date = new_doc_date
-        old_doc.drop_cache()
         job = self.__main_win.job_factories['index_updater'].make(
             self.__main_win.docsearch,
             new_docs={old_doc},
@@ -1350,7 +1490,7 @@ class DocPropertiesPanel(object):
             reload_list=True
         )
         self.__main_win.schedulers['main'].schedule(job)
-        self.__main_win.show_doc(old_doc, force_refresh=True)
+        self.__main_win.show_doc(old_doc)
 
     def _clear_label_list(self):
         self.widgets['labels'].freeze_child_notify()
@@ -1401,7 +1541,9 @@ class DocPropertiesPanel(object):
                 )
                 delete_button.set_relief(Gtk.ReliefStyle.NONE)
                 delete_button.connect("clicked", self.on_label_button_clicked)
-                ActionDeleteLabel(self.__main_win, self).connect([delete_button])
+                ActionDeleteLabel(self.__main_win, self).connect(
+                    [delete_button]
+                )
                 label_box.add(delete_button)
 
                 rowbox = Gtk.ListBoxRow()
@@ -1464,7 +1606,6 @@ class DocPropertiesPanel(object):
 
     def refresh_label_list(self):
         all_labels = sorted(self.__main_win.docsearch.label_list)
-        current_labels = sorted(self.labels.keys())
         self._clear_label_list()
         self._readd_label_widgets(all_labels)
         for label in self.labels:
