@@ -167,7 +167,7 @@ class SpinnerAnimation(Animation):
     src_size = 48
     ICON_SIZE = 64
 
-    layer = Drawer.PROGRESSION_INDICATOR_LAYER
+    layer = Drawer.SPINNER
 
     def __init__(self, position):
         Animation.__init__(self)
@@ -175,7 +175,7 @@ class SpinnerAnimation(Animation):
         self.position = position
         self.size = (self.ICON_SIZE, self.ICON_SIZE)
 
-        img = load_image("waiting.png")
+        img = load_image("waiting.png", pkg="paperwork.frontend")
         factor = self.ICON_SIZE / self.src_size
         img = img.resize((int(img.size[0] * factor), int(img.size[1] * factor)),
                          PIL.Image.ANTIALIAS)
